@@ -53,10 +53,10 @@ def project(app, request, project_name):
         raise NotFound("{} does not exist".format(project_name))
 
     # Generate the Package URLs for the packages we've hosted
-    file_urls = app.models.packaging.get_file_urls(project.name)
+    file_urls = app.models.packaging.get_file_urls(project)
 
     # Determine what the hosting mode is for this package
-    hosting_mode = app.models.packaging.get_hosting_mode(project.name)
+    hosting_mode = app.models.packaging.get_hosting_mode(project)
 
     project_urls = []
     if hosting_mode in {"pypi-scrape-crawl", "pypi-scrape"}:
@@ -65,7 +65,7 @@ def project(app, request, project_name):
         download_rel = "{}download".format(rel_prefix)
 
         # Generate the Homepage and Download URL links
-        release_urls = app.models.packaging.get_release_urls(project.name)
+        release_urls = app.models.packaging.get_release_urls(project)
         for version, (home_page, download_url) in release_urls.items():
             if home_page and home_page != "UNKNOWN":
                 project_urls.append({
@@ -82,7 +82,7 @@ def project(app, request, project_name):
                 })
 
     # Fetch the explicitly provided URLs
-    external_urls = app.models.packaging.get_external_urls(project.name)
+    external_urls = app.models.packaging.get_external_urls(project)
 
     resp = render_response(
         app, request,
@@ -94,13 +94,13 @@ def project(app, request, project_name):
     )
 
     # Add a header that points to the last serial
-    serial = app.models.packaging.get_last_serial(project.name)
+    serial = app.models.packaging.get_last_serial(project)
     resp.headers.add("X-PyPI-Last-Serial", serial)
 
     # Add a Link header to point at the canonical URL
     can_url = url_for(
         request, "warehouse.legacy.simple.project",
-        project_name=project.name,
+        project_name=project,
         _force_external=True,
     )
     resp.headers.add("Link", "<" + can_url + ">", rel="canonical")
@@ -133,7 +133,7 @@ def package(app, request, path):
     # Get the project name and normalize it
     lookup_filename = filename[:-4] if filename.endswith(".asc") else filename
     project = app.models.packaging.get_project_for_filename(lookup_filename)
-    normalized = normalize(project.name)
+    normalized = normalize(project)
 
     # Get the MD5 hash of the file
     content_md5 = app.models.packaging.get_filename_md5(filename)
@@ -149,7 +149,7 @@ def package(app, request, path):
     })
 
     # Look up the last serial for this file
-    serial = app.models.packaging.get_last_serial(project.name)
+    serial = app.models.packaging.get_last_serial(project)
     if serial is not None:
         headers["X-PyPI-Last-Serial"] = serial
 
